@@ -42,11 +42,8 @@
 
 // macro to generate a default constructor implementation
 #define kmime_mk_trivial_ctor( subclass, baseclass )                  \
-    subclass::subclass()           \
-    {                                                                     \
-    }                                                                     \
-    \
-    subclass::~subclass() {}
+    subclass::subclass() = default;          \
+    subclass::~subclass() = default;
 
 // end kmime_mk_trivial_ctor
 
@@ -231,16 +228,6 @@ void Structured::fromUnicodeString(const QString &s)
 
 //-----</Structured>-------------------------
 
-//-----<Address>-------------------------
-
-Address::Address() : Structured(new AddressPrivate)
-{
-}
-
-kmime_mk_dptr_ctor(Address, Structured)
-
-    Address::~Address() = default;
-
 // helper method used in AddressList and MailboxList
 static bool stringToMailbox(const QByteArray &address,
                             const QString &displayName, Types::Mailbox &mbox)
@@ -258,12 +245,10 @@ static bool stringToMailbox(const QByteArray &address,
     return true;
 }
 
-//-----</Address>-------------------------
-
 //-----<MailboxList>-------------------------
 
-kmime_mk_trivial_ctor_with_dptr(MailboxList, Address)
-kmime_mk_dptr_ctor(MailboxList, Address)
+kmime_mk_trivial_ctor_with_dptr(MailboxList, Structured)
+kmime_mk_dptr_ctor(MailboxList, Structured)
 
 QByteArray MailboxList::as7BitString(bool withHeaderType) const
 {
@@ -402,21 +387,66 @@ bool MailboxList::parse(const char *&scursor, const char *const send,
 //-----<SingleMailbox>-------------------------
 
 //@cond PRIVATE
-kmime_mk_trivial_ctor_with_dptr(SingleMailbox, MailboxList)
+kmime_mk_trivial_ctor_with_dptr(SingleMailbox, Structured)
 //@endcond
 
-bool SingleMailbox::parse(const char *&scursor, const char *const send,
-                          bool isCRLF)
+QByteArray SingleMailbox::as7BitString(bool withHeaderType) const
 {
-    Q_D(MailboxList);
-    if (!MailboxList::parse(scursor, send, isCRLF)) {
-        return false;
+    const Q_D(SingleMailbox);
+    if (isEmpty()) {
+        return {};
     }
 
-    if (d->mailboxList.count() > 1) {
-        KMIME_WARN << "multiple mailboxes in header allowing only a single one!"
-                   << Qt::endl;
+    QByteArray rv;
+    if (withHeaderType) {
+        rv = typeIntro();
     }
+    rv += d->mailbox.as7BitString(rfc2047Charset());
+    return rv;
+}
+
+void SingleMailbox::fromUnicodeString(const QString &s)
+{
+    Q_D(SingleMailbox);
+    from7BitString(encodeRFC2047Sentence(s, rfc2047Charset()));
+}
+
+QString SingleMailbox::asUnicodeString() const
+{
+    Q_D(const SingleMailbox);
+    return d->mailbox.prettyAddress();
+}
+
+bool SingleMailbox::isEmpty() const
+{
+    Q_D(const SingleMailbox);
+    return !d->mailbox.hasAddress() && !d->mailbox.hasName();
+}
+
+Types::Mailbox SingleMailbox::mailbox() const
+{
+    Q_D(const SingleMailbox);
+    return d->mailbox;
+}
+
+void SingleMailbox::setMailbox(const Types::Mailbox &mailbox)
+{
+    Q_D(SingleMailbox);
+    d->mailbox = mailbox;
+}
+
+bool SingleMailbox::parse(const char *&scursor, const char *const send, bool isCRLF)
+{
+    Q_D(SingleMailbox);
+    Types::Mailbox maybeMailbox;
+    if (!parseMailbox(scursor, send, maybeMailbox)) {
+        return false;
+    }
+    eatCFWS(scursor, send, isCRLF);
+    if (scursor != send) {
+        return false;
+    }
+    d->mailbox = maybeMailbox;
     return true;
 }
 
@@ -425,8 +455,8 @@ bool SingleMailbox::parse(const char *&scursor, const char *const send,
 //-----<AddressList>-------------------------
 
 //@cond PRIVATE
-kmime_mk_trivial_ctor_with_dptr(AddressList, Address)
-kmime_mk_dptr_ctor(AddressList, Address)
+kmime_mk_trivial_ctor_with_dptr(AddressList, Structured)
+kmime_mk_dptr_ctor(AddressList, Structured)
 //@endcond
 
 QByteArray AddressList::as7BitString(bool withHeaderType) const
@@ -844,8 +874,8 @@ bool Parametrized::parse(const char  *&scursor, const char *const send,
 //-----<Ident>-------------------------
 
 //@cond PRIVATE
-kmime_mk_trivial_ctor_with_dptr(Ident, Address)
-kmime_mk_dptr_ctor(Ident, Address)
+kmime_mk_trivial_ctor_with_dptr(Ident, Structured)
+kmime_mk_dptr_ctor(Ident, Structured)
 //@endcond
 
 QByteArray Ident::as7BitString(bool withHeaderType) const
@@ -891,7 +921,6 @@ bool Ident::parse(const char *&scursor, const char *const send, bool isCRLF)
     // msg-id   := angle-addr
 
     d->msgIdList.clear();
-    d->cachedIdentifier.clear();
 
     while (scursor != send) {
         eatCFWS(scursor, send, isCRLF);
@@ -942,7 +971,6 @@ void Ident::fromIdent(const Ident* ident)
 {
     d_func()->encCS = ident->d_func()->encCS;
     d_func()->msgIdList = ident->d_func()->msgIdList;
-    d_func()->cachedIdentifier = ident->d_func()->cachedIdentifier;
 }
 
 void Ident::appendIdentifier(const QByteArray &id)
@@ -969,51 +997,93 @@ void Ident::appendIdentifier(const QByteArray &id)
 //-----<SingleIdent>-------------------------
 
 //@cond PRIVATE
-kmime_mk_trivial_ctor_with_dptr(SingleIdent, Ident)
-kmime_mk_dptr_ctor(SingleIdent, Ident)
+kmime_mk_trivial_ctor_with_dptr(SingleIdent, Structured)
+kmime_mk_dptr_ctor(SingleIdent, Structured)
 //@endcond
 
-QByteArray SingleIdent::identifier() const
+QByteArray SingleIdent::as7BitString(bool withHeaderType) const
 {
-    if (d_func()->msgIdList.isEmpty()) {
+    Q_D(const SingleIdent);
+    if (d->msgId.isEmpty()) {
       return {};
     }
 
-    if (d_func()->cachedIdentifier.isEmpty()) {
-        const Types::AddrSpec &addr = d_func()->msgIdList.first();
-        if (!addr.isEmpty()) {
-            const QString asString = addr.asString();
-            if (!asString.isEmpty()) {
-                d_func()->cachedIdentifier = asString.toLatin1();// FIXME: change parsing to use QByteArrays
-            }
+    QByteArray rv;
+    if (withHeaderType) {
+        rv = typeIntro();
+    }
+    const QString asString = d->msgId.asString();
+    rv += '<';
+    if (!asString.isEmpty()) {
+        rv += asString.toLatin1(); // FIXME: change parsing to use QByteArrays
+    }
+    rv += '>';
+    return rv;
+}
+
+bool SingleIdent::isEmpty() const
+{
+    Q_D(const SingleIdent);
+    return d->msgId.isEmpty();
+}
+
+QByteArray SingleIdent::identifier() const
+{
+    Q_D(const SingleIdent);
+    if (d->msgId.isEmpty()) {
+        return {};
+    }
+    if (d->cachedIdentifier.isEmpty()) {
+        const QString asString = d->msgId.asString();
+        if (!asString.isEmpty()) {
+            d->cachedIdentifier = asString.toLatin1();// FIXME: change parsing to use QByteArrays
         }
     }
 
-    return d_func()->cachedIdentifier;
+    return d->cachedIdentifier;
 }
 
 void SingleIdent::setIdentifier(const QByteArray &id)
 {
     Q_D(SingleIdent);
-    d->msgIdList.clear();
+    d->msgId = {};
     d->cachedIdentifier.clear();
-    appendIdentifier(id);
+
+    QByteArray tmp = id;
+    if (!tmp.startsWith('<')) {
+        tmp.prepend('<');
+    }
+    if (!tmp.endsWith('>')) {
+        tmp.append('>');
+    }
+    AddrSpec msgId;
+    const char *cursor = tmp.constData();
+    if (parseAngleAddr(cursor, cursor + tmp.length(), msgId)) {
+        d->msgId = msgId;
+    } else {
+        qCWarning(KMIME_LOG) << "Unable to parse address spec!";
+    }
 }
 
-bool SingleIdent::parse(const char *&scursor, const char *const send,
-                        bool isCRLF)
+bool SingleIdent::parse(const char *&scursor, const char *const send, bool isCRLF)
 {
     Q_D(SingleIdent);
-    if (!Ident::parse(scursor, send, isCRLF)) {
+
+    d->msgId = {};
+    d->cachedIdentifier.clear();
+
+    AddrSpec maybeMsgId;
+    if (!parseAngleAddr(scursor, send, maybeMsgId, isCRLF)) {
         return false;
     }
-
-    if (d->msgIdList.count() > 1) {
-        KMIME_WARN << "more than one msg-id in header "
-                   << "allowing only a single one!"
-                   << Qt::endl;
+    eatCFWS(scursor, send, isCRLF);
+    // header end ending the list: OK.
+    if (scursor == send) {
+        d->msgId = maybeMsgId;
+        return true;
     }
-    return true;
+
+    return false;
 }
 
 //-----</SingleIdent>-------------------------
@@ -1023,7 +1093,7 @@ bool SingleIdent::parse(const char *&scursor, const char *const send,
 //-----<ReturnPath>-------------------------
 
 //@cond PRIVATE
-kmime_mk_trivial_ctor_with_name_and_dptr(ReturnPath, Generics::Address, Return-Path)
+kmime_mk_trivial_ctor_with_name_and_dptr(ReturnPath, Generics::Structured, Return-Path)
 //@endcond
 
 QByteArray ReturnPath::as7BitString(bool withHeaderType) const
@@ -1741,7 +1811,7 @@ bool ContentID::parse(const char *&scursor, const char *const send, bool isCRLF)
     const char *origscursor = scursor;
     if (!SingleIdent::parse(scursor, send, isCRLF)) {
         scursor = origscursor;
-        d->msgIdList.clear();
+        d->msgId = {};
         d->cachedIdentifier.clear();
 
         while (scursor != send) {
@@ -1782,7 +1852,7 @@ bool ContentID::parse(const char *&scursor, const char *const send, bool isCRLF)
             // /Almost parseAngleAddr
 
             maybeContentId.localPart = QString::fromLatin1(result); // FIXME: just use QByteArray instead of AddrSpec in msgIdList?
-            d->msgIdList.append(maybeContentId);
+            d->msgId = maybeContentId;
 
             eatCFWS(scursor, send, isCRLF);
             // header end ending the list: OK.
